import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

class TerminalImage {
  String? previewUrl;
  String? imageUrl;
  String? coverImageUrl;

  TerminalImage({
    this.previewUrl,
    this.imageUrl,
    this.coverImageUrl,
  });

  factory TerminalImage.fromJson(Map<String, dynamic> map) {
    return TerminalImage(
      previewUrl: map['PreviewUrl'],
      imageUrl: map['ImageUrl'],
      coverImageUrl: map['CoverImageUrl'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (previewUrl != null) map['PreviewUrl'] = previewUrl;
    if (imageUrl != null) map['ImageUrl'] = imageUrl;
    if (coverImageUrl != null) map['CoverImageUrl'] = coverImageUrl;
    return map;
  }
}

class QuickFWidget extends StatelessWidget implements FWidget {
  const QuickFWidget(this.child, {Key? key}) : super(key: key);
  final Widget child;
  @override
  Widget build(BuildContext context) => child;
}

/// 所有图片的入口
class FContentImage extends StatelessWidget implements FWidget {
  /// 图片参数
  final RemedicalInfoDTO? remedicalInfo;

  /// 是否是测量页面
  final bool? isMeasure;

  /// 改变图片地址
  final VoidCallback? onDoubleTap;

  /// 图片单机事件
  final VoidCallback? onTap;

  /// AI 良恶性
  late final DiagnosisConclusionEnum diagnosisConclusion =
      remedicalInfo?.diagnosisConclusion ?? DiagnosisConclusionEnum.NotRequired;

  /// 图片类型
  late RemedicalFileDataTypeEnum? fileDataType;

  /// 图片地址
  late TerminalImage? terminalImage;

  /// AI测量的器官
  late final List<DiagnosisOrganEnum>? diagnosisOrgans =
      remedicalInfo?.diagnosisOrgans ?? [DiagnosisOrganEnum.Null];

  bool get isCarotidResult => remedicalInfo?.carotidResult != null;

  /// 判断是否有AI器官
  bool get hasDiagnosisOrgans => diagnosisOrgans?.length != 0;

  /// 判断是否展示ai字
  bool get isShowDiagnosisOrgans => [
        DiagnosisConclusionEnum.NoObviousLesion,
        DiagnosisConclusionEnum.Benign,
        DiagnosisConclusionEnum.Malignant,
        DiagnosisConclusionEnum.BenignAndMalignant
      ].contains(diagnosisConclusion);

  /// 是否是选择状态
  final bool isSelectStutas;

  FContentImage({
    Key? key,
    this.terminalImage,
    this.fileDataType,
    this.remedicalInfo,
    this.isMeasure = false,
    this.onDoubleTap,
    this.onTap,
    this.isSelectStutas = false,
  }) : super(key: key);

  @override
  FWidget build(BuildContext context) {
    if (terminalImage == null) {
      terminalImage = new TerminalImage(
          previewUrl: remedicalInfo?.terminalImages?.previewUrl ?? '',
          imageUrl: remedicalInfo?.terminalImages?.imageUrl ?? '',
          coverImageUrl: remedicalInfo?.terminalImages?.coverImageUrl ?? '');
    }
    if (fileDataType == null) {
      fileDataType = remedicalInfo?.fileDataType;
    }

    if (isSelectStutas) {
      return FContainer(
        width: 190,
        height: 160,
        color: Colors.black,
        child: FStack(
          children: [
            FCenter(
              child: FImage.network(
                terminalImage!.previewUrl!,
                errorBuilder: ((context, error, stackTrace) {
                  return Container(
                    child: Text('error'),
                  );
                }),
              ),
            ),
          ],
        ),
      );
    } else
      return _buildFContentImage(
        fileDataType!,
        terminalImage!.previewUrl!,
      );
  }

  FWidget _buildFContentImage(
    RemedicalFileDataTypeEnum fileDataType,
    String previewUrl,
  ) {
    switch (fileDataType) {
      case RemedicalFileDataTypeEnum.Image:
      case RemedicalFileDataTypeEnum.ThirdVidSingle:
      case RemedicalFileDataTypeEnum.VinnoVidSingle:
        return _buildImageCard(
          previewUrl,
        );
      case RemedicalFileDataTypeEnum.ThirdVidMovie:
      case RemedicalFileDataTypeEnum.VinnoVidMovie:
        return _buildVidMovieCard(
          previewUrl,
        );
      default:
        return FContainer(
          child: FText('error'),
        );
    }
  }

  FWidget _buildLeftAIOrCarotidLogo() {
    if (hasDiagnosisOrgans) {
      return FPositioned(
        bottom: 5,
        left: 5,
        child: _buildAIDiagnosticOrgans(),
      );
    }
    if (isCarotidResult) {
      return FPositioned(
        bottom: 5,
        left: 5,
        child: FIcon(
          FIcons.arteria_carotis,
          color: Colors.lightBlue,
        ),
      );
    }
    return FPositioned(
      bottom: 5,
      left: 5,
      child: FText(
        remedicalInfo?.application ?? "",
        style: TextStyle(
          color: Colors.white,
          fontSize: 8,
          overflow: TextOverflow.ellipsis,
        ),
      ),
    );
  }

  FWidget _buildRigthAIText() {
    if (hasDiagnosisOrgans) {
      return FPositioned(
        bottom: 5,
        right: 5,
        child: QuickFWidget(
          Tooltip(
            textStyle: TextStyle(
              fontSize: 16,
              color: Colors.white,
            ),
            message: _buildAIText(diagnosisConclusion),
            child: FText(
              'AI',
              style: TextStyle(
                color: _buildAITextColor(
                  diagnosisConclusion,
                ),
                fontSize: 20,
              ),
            ),
          ),
        ),
      );
    }

    if (isCarotidResult) {
      return FPositioned(
        bottom: 5,
        right: 5,
        child: FIcon(
          FIcons.three_dimensional,
          color: Colors.lightBlue,
        ),
      );
    }
    return FSizedBox();
  }

  FWidget _buildVidMovieCard(
    String previewUrl,
  ) {
    return FContainer(
      width: 190,
      height: 160,
      color: Colors.black,
      child: FStack(
        children: [
          FCenter(
            child: FImage.network(
              previewUrl,
              errorBuilder: ((context, error, stackTrace) {
                return Container(
                  child: Text('error'),
                );
              }),
            ),
          ),
          _buildLeftAIOrCarotidLogo(),
          _buildRigthAIText(),
          FInkWell(
            onDoubleTap: () {
              onDoubleTap?.call();
            },
            onTap: () {
              onTap?.call();
            },
            child: Center(
              child: Container(
                child: Icon(
                  Icons.play_circle_outline_rounded,
                  color: Colors.white,
                  size: 50,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  FWidget _buildImageCard(
    String previewUrl,
  ) {
    return FInkWell(
      onDoubleTap: () {
        onDoubleTap?.call();
      },
      onTap: () {
        onTap?.call();
      },
      child: Container(
        width: 190,
        height: 160,
        color: Colors.black,
        child: Stack(
          children: [
            Center(
              child: Image.network(
                previewUrl,
                errorBuilder: ((context, error, stackTrace) {
                  return Container(
                    child: Text('error'),
                  );
                }),
              ),
            ),
            _buildLeftAIOrCarotidLogo(),
            _buildRigthAIText(),
          ],
        ),
      ),
    );
  }

  String _buildAIText(DiagnosisConclusionEnum diagnosisConclusion) {
    switch (diagnosisConclusion) {
      case DiagnosisConclusionEnum.NotRequired:
        return i18nBook.remedical.noNeedAIYet.t;
      case DiagnosisConclusionEnum.InProcess:
        return i18nBook.remedical.didNotHandle.t;
      case DiagnosisConclusionEnum.Unrecognized:
        return i18nBook.remedical.didNotIdentify.t;
      case DiagnosisConclusionEnum.NoObviousLesion:
        return i18nBook.remedical.noAbnormal.t;
      case DiagnosisConclusionEnum.Benign:
        return i18nBook.remedical.benign.t;
      case DiagnosisConclusionEnum.Malignant:
        return i18nBook.remedical.malignant.t;
      case DiagnosisConclusionEnum.BenignAndMalignant:
        return i18nBook.remedical.benignWithMalignant.t;
      default:
        return i18nBook.remedical.didNotIdentify.t;
    }
  }

  dynamic _buildAITextColor(DiagnosisConclusionEnum diagnosisConclusion) {
    switch (diagnosisConclusion) {
      case DiagnosisConclusionEnum.NoObviousLesion:
        return Colors.lightBlue;
      case DiagnosisConclusionEnum.Benign:
        return Colors.greenAccent;
      case DiagnosisConclusionEnum.Malignant:
        return Colors.redAccent;
      case DiagnosisConclusionEnum.BenignAndMalignant:
        return Colors.orangeAccent;
      default:
        return Colors.transparent;
    }
  }

  FWidget _buildAIDiagnosticOrgans() {
    switch (diagnosisOrgans?[0]) {
      case DiagnosisOrganEnum.Null:
        return const FSizedBox();
      case DiagnosisOrganEnum.placeHolder_1:
        return const FSizedBox();
      case DiagnosisOrganEnum.Breast:
        return FIcon(
          FIcons.fis_breast,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Abdomen:
        return FIcon(
          FIcons.fis_abdomen,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );

      case DiagnosisOrganEnum.Liver:
        return FIcon(
          FIcons.fis_liver,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Cholecyst:
        return FIcon(
          FIcons.fis_gallbladder,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Kidney:
        return FIcon(
          FIcons.fis_kidney,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );
      case DiagnosisOrganEnum.Spleen:
        return FIcon(
          FIcons.fis_spleen,
          color: _buildAITextColor(
            diagnosisConclusion,
          ),
        );

      default:
        return FContainer();
    }
  }
}
